﻿using System;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Tcp;
using System.Windows.Forms;

using TestLibrary;

namespace TestServer
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form, IObserver
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Delegate
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 메시지 설정하기 대리자 - SetMessageDelegate(message)

        /// <summary>
        /// 메시지 설정하기 대리자
        /// </summary>
        /// <param name="message">메시지</param>
        private delegate void SetMessageDelegate(string message);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// TCP 서버 채널
        /// </summary>
        private TcpServerChannel tcpServerChannel = null;

        /// <summary>
        /// 메시지 설정하기 대리자
        /// </summary>
        private SetMessageDelegate setMessageDelegate = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 통지하기 - Notify(message)

        /// <summary>
        /// 통지하기
        /// </summary>
        /// <param name="message">메시지</param>
        public void Notify(string message)
        {
            Invoke(this.setMessageDelegate, message);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 폼 로드시 처리하기 - Form_Load(sender, e)

        /// <summary>
        /// 폼 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_Load(object sender, EventArgs e)
        {
            this.setMessageDelegate = new SetMessageDelegate(SetMessage);
        }

        #endregion
        #region 시작 버튼 클릭시 처리하기 - startButton_Click(sender, e)

        /// <summary>
        /// 시작 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void startButton_Click(object sender, EventArgs e)
        {
            this.serverNameTextBox.Enabled = false;
            this.portTextBox.Enabled       = false;

            this.tcpServerChannel = new TcpServerChannel(Convert.ToInt32(this.portTextBox.Text));

            ChannelServices.RegisterChannel(this.tcpServerChannel, true);

            RemotingConfiguration.RegisterWellKnownServiceType
            (
                typeof(RemotableObject),
                this.serverNameTextBox.Text,
                WellKnownObjectMode.Singleton
            );

            TestLibrary.Cache.Attach(this);

            this.statusToolStripStatusLabel.Text = "상태 : 채널(" + this.tcpServerChannel.ChannelName + "), 수신중...";
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 메시지 설정하기 - SetMessage(message)

        /// <summary>
        /// 메시지 설정하기
        /// </summary>
        /// <param name="message">메시지</param>
        private void SetMessage(string message)
        {
            this.messageTextBox.Text = message;
        }

        #endregion
    }
}